from autobahn.websocket.protocol import createWsUrl
from autobahn.twisted.websocket import WebSocketServerFactory
from autobahn.twisted.websocket import WebSocketClientFactory


class JRPCFactory(object):
    """
    Factory mixin for hosting a JRPC peer. Takes a JRPCProtocol.
    Supports passing a dispatch dictionary. If no dispatch
    is provided the protocol itself is used for gathering
    methods that begin with "do".
    """

    def __init__(self, host, *args, **kwargs):
        self.dispatch = kwargs.pop('dispatch', {})
        port = kwargs.pop('port', 9000)
        wsUrl = createWsUrl(host, port)
        super(JRPCFactory, self).__init__(wsUrl, *args, **kwargs)

    def buildProtocol(self, addr):
        p = self.protocol(self.dispatch)
        p.factory = self
        return p


class JRPCServerFactory(JRPCFactory, WebSocketServerFactory):
    pass


class JRPCClientFactory(JRPCFactory, WebSocketClientFactory):
    pass



import json


class JRPCRequest(object):
    """
    Represents the intention to invoke an RPC method whether
    locally or remotely. A request with an id of None will
    not recieve a result and will not increment request
    counters.
    """

    def __init__(self, method, args=None, kwargs=None, id=None):
        # the name of the method to invoke
        self.method = method
        # positional arguments
        self.args = args or tuple()
        # keyword arguments
        self.kwargs = kwargs or dict()
        # id representing the request
        self.id = id

    def data(self):
        '''return dictionary representation of the request'''
        return {
            'method': self.method,
            'args': self.args,
            'kwargs': self.kwargs,
            'id': self.id
        }

    def json(self):
        '''return json representation of the request'''
        return json.dumps(self.data())

    @classmethod
    def from_json(cls, json_str):
        '''create a JRPCRequest from json'''
        obj = json.loads(json_str)
        return cls(obj['method'], obj.get('args', tuple()),
                   obj.get('kwargs', dict()), obj.get('id', None))



#!/usr/bin/env python

from setuptools import setup

setup(name="jrpc",
      version="0.1.2",
      packages=['jrpc'],
      scripts=['bin/jrpc'],
      install_requires=[
          "Twisted",
          "autobahn",
      ],

      # metadata for upload to PyPI
      author="Dustin Lacewell",
      author_email="dlacewell@gmail.com",
      description="JSON RPC for Autobahn/Twisted",
      keywords="twisted autobahn rpc json",
      url="http://github.com/dustinlacewell/jrpc", )



from autobahn.twisted.websocket import WebSocketServerProtocol
from autobahn.twisted.websocket import WebSocketClientProtocol

from .manager import JRPCManager
from .dispatcher import Dispatcher


class JRPCProtocol(object):
    """
    This class is the main protocol for handling interaction with
    a remote JRPC peer. It features a simple interface for making
    both fire-and-forget invocations but also requests that will
    result in a response in the form of a deferred object.

    To provide callable methods for peers to invoke, subclasses
    can implement methods beginning with the token "do" which
    will make them automatically available.
    """

    def __init__(self, dispatch=None):
        super(JRPCProtocol, self).__init__()
        self.manager = JRPCManager(self, dispatch or Dispatcher(self))

    def onMessage(self, payload, isBinary):
        self.manager.handle(payload)

    def request(self, *args, **kwargs):
        return self.manager.call(*args, **kwargs)

    def invoke(self, *args, **kwargs):
        return self.manager.invoke(*args, **kwargs)


class JRPCServerProtocol(JRPCProtocol, WebSocketServerProtocol):
    pass


class JRPCClientProtocol(JRPCProtocol, WebSocketClientProtocol):
    pass
